import { Test, TestingModule } from "@nestjs/testing";
import { INestApplication } from "@nestjs/common";
import request from 'supertest'
import { AppModule } from "src/infrastructure/modules";

describe('E2E Testing of /papers', () => {
    let app: INestApplication;

    beforeAll(async () => {
        const moduleRef: TestingModule = await Test.createTestingModule({
            imports: [AppModule],
        }).compile();

        app = moduleRef.createNestApplication();
        await app.init();
    });
    
    it('GET /papers/{uuid} | Should return one exact item on page', async () => {
        const test = await request(app.getHttpServer())
            .get('/papers/eeeb2d01-8315-454e-b33f-3d6caa25db42') // ??? Fetch a random object from DB
            .expect(200);
        
        // Checking received data
        expect(test.body.data).toBeDefined();
        
        expect(test.body.data.length).toBe(1);
        expect(test.body.data[0].id).toBeDefined();
        expect(test.body.data[0].title).toBeDefined();
        expect(test.body.data[0].authors).toBeDefined();
        expect(test.body.data[0].summary).toBeDefined();
        expect(test.body.data[0].tags).toBeDefined();
        expect(test.body.data[0].content).toBeDefined();
        expect(test.body.data[0].id).toBe('eeeb2d01-8315-454e-b33f-3d6caa25db42');

        // Checking received meta
        expect(test.body.meta).toBeDefined();

        expect(test.body.meta.total).toBeDefined();
        expect(test.body.meta.pagenum).toBeDefined();
        expect(test.body.meta.order).toBeDefined();
        expect(test.body.meta.pagesize).toBeDefined();
        expect(test.body.meta.hasNext).toBeDefined();
        expect(test.body.meta.hasPrev).toBeDefined();
        expect(test.body.meta.total).toBe(1);
        expect(test.body.meta.pagenum).toBe(1);
    });

    it('GET /papers/search? | Should return multiple items', async () => {
        const test = await request(app.getHttpServer())
            .get('/papers/search?query=at&page=1')
            .expect(200);

        // Checking received data
        expect(test.body.data).toBeDefined();

        expect(test.body.data.length).toBeGreaterThan(0);
        for (const paper of test.body.data) {
            expect(paper.id).toBeDefined();
            expect(paper.title).toBeDefined();
            expect(paper.authors).toBeDefined();
            expect(paper.summary).toBeDefined();
            expect(paper.tags).toBeDefined();
            expect(paper.content).toBeDefined();
        }
        
        // Checking received meta
        expect(test.body.meta).toBeDefined();

        expect(test.body.meta.total).toBeDefined();
        expect(test.body.meta.pagenum).toBeDefined();
        expect(test.body.meta.order).toBeDefined();
        expect(test.body.meta.pagesize).toBeDefined();
        expect(test.body.meta.hasNext).toBeDefined();
        expect(test.body.meta.hasPrev).toBeDefined();
        expect(test.body.meta.total).toBeGreaterThan(0);
        expect(test.body.meta.pagenum).toBe(1);
    });

    afterAll(async () => {
        await app.close();
    })
});